from searchable_collection.simple_comparator import SimpleComparator
from searchable_collection.item import SearchableItem
from searchable_collection.collection import SearchableCollection
__version__ = "0.0.3"